#!/bin/bash

programs=(hello-world sequence-test \
  sequence-sgn-test sequence-sgn-test2 \
  assoc-test assoc-test2 objects1 objects2 module-test1 \
  statprof-demo command-line-demo \
  orientation1 orientation2 \
  iterator-example nonpure-iterator-example \
  bits fold)

for fl in ${programs[@]} ; do
    ../../tools/compute-theme-d-program-deps.scm $fl.thp examples
done
